{ This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  ---------------------------------------------------------------------------

    Author : Luc DAVID Email: luckylazarus@free.fr
    2007-2008
    Last update : 01/06/2008

  --------------------------------------------------------------------------- }

unit SqlitePassChooseDatasetDialog;
{$IFDEF FPC}
 {$mode DELPHI}
{$ENDIF}

interface

uses
 {$IFDEF FPC}
  LResources,
 {$ENDIF}
 {$IFDEF WIN32}
  Windows,
 {$ENDIF}
  Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, DB, ComCtrls, SqlitePassDbo, ImgList;

type

  { TSqlitePassChooseDatasetDialog }

  TSqlitePassChooseDatasetDialog = class(TForm)
    PanelBottom: TPanel;
    BtOk: TButton;
    BtCancel: TButton;
    PanelTop: TPanel;
    LabelAvailableDatasets: TLabel;
    Tv: TTreeView;
    TreeViewImageList: TImageList;
    procedure TvDblClick(Sender: TObject);
   Public
   constructor Create(AOwner: TComponent; Dataset: TSqlitePassDataset); reintroduce;
  end;

var
  SqlitePassChooseDatasetDlg: TSqlitePassChooseDatasetDialog;

implementation

{$IFNDEF FPC}
 {$R *.DFM}
{$ENDIF}

constructor TSqlitePassChooseDatasetDialog.Create(AOwner: TComponent;
  Dataset: TSqlitePassDataset);

 function AddNode(ParentNode: TTreeNode; Caption: String; ImageIndex, SelectedImageIndex: Integer): TTreeNode;
   begin
   if Assigned(ParentNode)
      then Result := Tv.Items.AddChild(ParentNode,Caption)
      else Result := Tv.Items.Add(nil,'Database');
   Result.ImageIndex := ImageIndex;
   Result.SelectedIndex := SelectedImageIndex;
   end;

const
EmptyImageIndex = -1;
SelectedImageIndex = 1;
DatabaseImageIndex = 0;
TableImageIndex = 2;
QueryImageIndex = 3;
WiewImageIndex = 4;
TriggerImageIndex = 5;
SQLImageIndex = 6;
SQLItemImageIndex = 7;
SystemTableImageIndex = 8;
ItemImageIndex = 9;
var
i: Integer;
TvItem: TTreeNode;
DatabaseNode, TablesNode, QueriesNode, ViewsNode, SystemTablesNode,
SQLNode, SQLCreateNode, SQLCreateTableNode, SQLSelectNode, SQLInsertNode,
SQLUpdateNode, SQLDeleteNode, SQLOthersNode: TTreeNode;
Db: TSqlitePassDatabase;
begin
inherited Create(AOwner);
if Dataset = nil then exit;
Db := Dataset.Database;
if Db = nil then exit;

Try
Tv.Items.BeginUpdate;

Tv.Items.Clear;
DatabaseNode := nil;
TablesNode   := nil;
QueriesNode  := nil;
ViewsNode    := nil;
SQLNode := nil;
SQLCreateNode := nil;
SQLCreateTableNode := nil;
SQLSelectNode := nil;
SQLInsertNode := nil;
SQLUpdateNode := nil;
SQLDeleteNode := nil;
SQLOthersNode := nil;
        
DatabaseNode := AddNode(nil,'Database',DatabaseImageIndex,DatabaseImageIndex);
TablesNode   := AddNode(DatabaseNode,'Tables',TableImageIndex,TableImageIndex);
QueriesNode  := AddNode(DatabaseNode,'Queries',QueryImageIndex,QueryImageIndex);
ViewsNode    := AddNode(DatabaseNode,'Views',WiewImageIndex,WiewImageIndex);
SQLNode := AddNode(DatabaseNode,'SQL',SQLImageIndex,SQLImageIndex);
SystemTablesNode := AddNode(DatabaseNode,'System Tables',SystemTableImageIndex,SystemTableImageIndex);

for i:= 0 to Pred(Db.TableDefs.Count)
    do if Not Db.IsSystemTable(Db.TableDefs[i].TableName)
       then AddNode(TablesNode,Db.TableDefs[i].TableFullName,ItemImageIndex,SelectedImageIndex)
       else AddNode(SystemTablesNode, Db.TableDefs[i].TableFullName,ItemImageIndex,SelectedImageIndex);

for i:= 0 to Pred(Db.QueryDefs.Count)
    do AddNode(QueriesNode,Db.QueryDefs[i].QueryFullName,ItemImageIndex, SelectedImageIndex);

for i:= 0 to Pred(Db.Views.Count)
    do AddNode(ViewsNode,Db.Views[i].ViewName,ItemImageIndex, SelectedImageIndex);

for i:= 0 to Pred(Db.SQLStmts.Count)
    do begin
       case Db.SQLStmts[i].SQLType of
            stSelect: begin
                      if Not Assigned(SQLSelectNode)
                         then SQLSelectNode := AddNode(SQLNode,'(Select)',SQLItemImageIndex,SelectedImageIndex);
                      AddNode(SQLSelectNode,Db.SQLStmts[i].StmtName,ItemImageIndex,SelectedImageIndex);
                      end;
            stCreate: begin
                      if Not Assigned(SQLCreateNode)
                         then SQLCreateNode:= AddNode(SQLNode,'(Create)',SQLItemImageIndex,SelectedImageIndex);
                      AddNode(SQLCreateNode,Db.SQLStmts[i].StmtName,ItemImageIndex,SelectedImageIndex);
                      end;
            stCreateTable: begin
                      if Not Assigned(SQLCreateTableNode)
                         then SQLCreateTableNode := AddNode(SQLNode,'(Create Table)',SQLItemImageIndex,SelectedImageIndex);
                      AddNode(SQLCreateTableNode,Db.SQLStmts[i].StmtName,ItemImageIndex,SelectedImageIndex);
                      end;
            stInsert: begin
                      if Not Assigned(SQLInsertNode)
                         then SQLInsertNode := AddNode(SQLNode,'(Insert)',SQLItemImageIndex,SelectedImageIndex);
                      AddNode(SQLInsertNode,Db.SQLStmts[i].StmtName,ItemImageIndex,SelectedImageIndex);
                      end;
            stUpdate: begin
                      if Not Assigned(SQLUpdateNode)
                         then SQLUpdateNode := AddNode(SQLNode,'(Update)',SQLItemImageIndex,SelectedImageIndex);
                      AddNode(SQLUpdateNode,Db.SQLStmts[i].StmtName,ItemImageIndex,SelectedImageIndex);
                      end;
            stDelete: begin
                      if Not Assigned(SQLDeleteNode)
                         then SQLDeleteNode := AddNode(SQLNode,'(Delete)',SQLItemImageIndex,SelectedImageIndex);
                      AddNode(SQLDeleteNode,Db.SQLStmts[i].StmtName,ItemImageIndex,SelectedImageIndex);
                      end;
            else      begin
                      if Not Assigned(SQLOthersNode)
                         then SQLOthersNode := AddNode(SQLNode, '(Others)',SQLItemImageIndex,SelectedImageIndex);
                      AddNode(SQLOthersNode, Db.SQLStmts[i].StmtName,ItemImageIndex,SelectedImageIndex);
                      end;
            end; { Case }
       end;
finally
Tv.FullExpand;
if Tv.Items.Count > 0
   then Tv.TopItem := DatabaseNode;
Tv.Items.EndUpdate;
end;
end;

procedure TSqlitePassChooseDatasetDialog.TvDblClick(Sender: TObject);
begin
ModalResult := mrOk;
end;

initialization
 {$IFDEF FPC}
  {$I SqlitePassSortByDialog.lrs}
 {$ENDIF}
end.
