unit SqlitePassCreateNewDatabaseDialog;

interface

uses
 {$IFDEF FPC}
  LResources,
 {$ENDIF}
 {$IFDEF WIN32}
  Windows,
 {$ENDIF}
  Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons, SqlitePassDbo;

type
  TSqlitePassCreateNewDatabaseDlg = class(TForm)
    PanelButtons: TPanel;
    ButtonOk: TButton;
    ButtonCancel: TButton;
    Label1: TLabel;
    CbCheckFileExt: TCheckBox;
    TEditDbFileFullPath: TEdit;
    LabelDbFileFullPath: TLabel;
    SbGetDirectory: TSpeedButton;
    CbDbEncoding: TComboBox;
    Label4: TLabel;
    Label5: TLabel;
    CbDbPageSize: TComboBox;
    CbDbKind: TComboBox;
    CbDbStorage: TComboBox;
    Label6: TLabel;
    Label7: TLabel;
    CbDbVacuumType: TComboBox;
    CbLegacyFileFormat: TCheckBox;
    OpenDialog: TOpenDialog;
    Shape6: TShape;
    Shape1: TShape;
    Label2: TLabel;
    Label3: TLabel;
    Shape2: TShape;
    Shape3: TShape;
    Label8: TLabel;
    procedure ButtonOkClick(Sender: TObject);
    procedure CbDbStorageClick(Sender: TObject);
    procedure SbGetDirectoryClick(Sender: TObject);
    procedure TEditDbFileFullPathChange(Sender: TObject);
    procedure CbDbKindChange(Sender: TObject);
    procedure CbCheckFileExtClick(Sender: TObject);
  private
    Db: TSqlitePassDatabase;
    DatabaseType: TSqlitePassDatabaseType;
    Procedure CheckFileExtension;
    function GetFileExt: String;
  public
    constructor Create(AOwner: TComponent); reintroduce;
  end;

var
  SqlitePassCreateNewDatabaseDlg: TSqlitePassCreateNewDatabaseDlg;

implementation

{$IFNDEF FPC}
 {$R *.DFM}
{$ENDIF}

constructor TSqlitePassCreateNewDatabaseDlg.Create(AOwner: TComponent);
begin
 Inherited Create(AOwner);
 Db := TSqlitePassDatabase.Create(AOwner);
 CbDbStorage.ItemIndex := 0;
 CbDbKind.ItemIndex := 3;
 CbDbEncoding.ItemIndex := 0;
 CbDbPageSize.ItemIndex := 3;
 CbDbVacuumType.ItemIndex := 0;
 TEditDbFileFullPath.Text := GetCurrentDir + '\NewDatabase.' + GetFileExt;
 CbDbStorageClick(Self);
 CbDbKindChange(Self);
end;

procedure TSqlitePassCreateNewDatabaseDlg.ButtonOkClick(Sender: TObject);
var
DbName: String;
Encoding: TSqlitePassEncoding;
AutoVacuum, PageSize: Integer;
begin

CheckFileExtension;

Case CbDbStorage.ItemIndex of
     0: DbName := TEditDbFileFullPath.Text;
     1: DbName := '';
     2: DbName := ':memory:';
     end;

Case CbDbEncoding.ItemIndex of
     0: Encoding := UTF_8;
     1: Encoding := UTF_16;
     2: Encoding := UTF_16le;
     3: Encoding := UTF_16be;
     end;

AutoVacuum := CbDbVacuumType.ItemIndex;
PageSize := StrToInt(CbDbPageSize.Text);

Db.CreateDatabase(DbName, DatabaseType, Encoding, PageSize, AutoVacuum);
end;


procedure TSqlitePassCreateNewDatabaseDlg.CbDbStorageClick(
  Sender: TObject);
var
IsStored: Boolean;
begin
IsStored := (CbDbStorage.ItemIndex = 0);
CbCheckFileExt.Enabled   := IsStored;
LabelDbFileFullPath.Enabled  := IsStored;
TEditDbFileFullPath.Enabled  := IsStored;
SbGetDirectory.Enabled   := IsStored;
end;

procedure TSqlitePassCreateNewDatabaseDlg.SbGetDirectoryClick(
  Sender: TObject);
begin
 OpenDialog.FileName := 'NewDatabase.' + GetFileExt;
 If OpenDialog.Execute
    then TEditDbFileFullPath.Text := OpenDialog.FileName;
end;


procedure TSqlitePassCreateNewDatabaseDlg.TEditDbFileFullPathChange(
  Sender: TObject);
begin
 ButtonOk.Enabled :=  (TEditDbFileFullPath.Text <> '');
end;

function TSqlitePassCreateNewDatabaseDlg.GetFileExt: String;
begin
 Case CbDbKind.ItemIndex of
     0: begin
        DatabaseType := dbtUnknown;
        Result := 'db';
        end;
     1: begin
        DatabaseType := dbtKexi;
        Result := 'kexi';
        end;
     2: begin
        DatabaseType := dbtSQLiteAdmin;
        Result := 's3db';
        end;
     3: begin
        DatabaseType := dbtSQLiteExpert;
        Result := 'db3';
        end;
     4: begin
        DatabaseType := dbtSQLite4Fpc;
        Result := 's3fpc';
        end;
     5: begin
        DatabaseType := dbtSQLitePass;
        Result := 'sp3';
        end;
     end;
end;

procedure TSqlitePassCreateNewDatabaseDlg.CbDbKindChange(Sender: TObject);
begin
 CheckFileExtension;
end;

procedure TSqlitePassCreateNewDatabaseDlg.CheckFileExtension;
var
Path, Ext: String;
begin
if CbCheckFileExt.Enabled and CbCheckFileExt.Checked then
   begin
   Path := TEditDbFileFullPath.Text;
   Ext := ExtractFileExt(Path);
   System.Delete(Path, Length(Path)-Length(Ext)+1, Length(Ext));
   Ext := GetFileExt;
   if Path[Length(Path)] <> '.'
      then Ext := '.' + Ext;
   TEditDbFileFullPath.Text := Path + Ext;
   end;
end;

procedure TSqlitePassCreateNewDatabaseDlg.CbCheckFileExtClick(
  Sender: TObject);
begin
 CheckFileExtension;
end;

initialization
 {$IFDEF FPC}
  {$I SqlitePassCreateNewDatabaseDialog.lrs}
 {$ENDIF}
end.
