{ This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  ---------------------------------------------------------------------------

    Author : Luc DAVID Email: luckylazarus@free.fr
    2007-2008
    Last update : 02-03-2008

  --------------------------------------------------------------------------- }

unit SqlitePassLocateDialog;
{$i SqlitePassDbo.inc}

{$IFDEF FPC}
 {.SqlitePassLocateDialog$mode DELPHI}
{$ENDIF}

interface

uses
 {$IFDEF FPC}
  Variants, LResources,
 {$ELSE}
  {$IFDEF Delphi6}
   Variants,
  {$ENDIF}
  Windows, Messages,
 {$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons, db, Math,
  SqlitePassDbo;

type
  TSqlitePassLocateDlg = class(TForm)
    PanelIndexesToolBar: TPanel;
    PanelIndexMain: TPanel;
    Splitter1: TSplitter;
    PanelAvailableIndexes: TPanel;
    ListBoxFields: TListBox;
    PanelSQL: TPanel;
    PanelIndexApplyChanges: TPanel;
    SbCancel: TSpeedButton;
    PanelCaptionAvailableIFields: TPanel;
    SbOk: TSpeedButton;
    Image5: TImage;
    Panel1: TPanel;
    CheckBoxUseQuotes: TCheckBox;
    Panel2: TPanel;
    Label1: TLabel;
    TEditKeyValues: TEdit;
    Label4: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    TEditKeyFields: TEdit;
    CbPartialKeys: TCheckBox;
    CbCaseInsensitive: TCheckBox;
    procedure SbOkClick(Sender: TObject);
    procedure SbCancelClick(Sender: TObject);
    procedure ListBoxFieldsDblClick(Sender: TObject);
  private
    MyDb: TSqlitePassDatabase;
    MyDataset: TSqlitePassDataset;
    PreviousFilter: String;
    procedure RefreshFieldsList;
    function ExtractStrValue(const Str: string; var Pos: Integer): string;
    function VarArrayFromStr(LocateStrValues: String): Variant;
  public
    constructor Create(AOwner: TComponent; Dataset: TSqlitePassDataset); reintroduce;
  end;

var
  SqlitePassLocateDlg: TSqlitePassLocateDlg;

implementation

uses RenameItem, CreateNewIndex;

{$IFNDEF FPC}
  {$R *.DFM}
{$ENDIF}


constructor TSqlitePassLocateDlg.Create(AOwner: TComponent; Dataset: TSqlitePassDataset);
begin
inherited Create(AOwner);

ListBoxFields.Clear;
if Dataset = nil
   then abort
   else begin
        MyDataset := Dataset;
        Caption := 'Locate Editor for ' + MyDataset.DatasetName;
        MyDb := MyDataset.Database;
        RefreshFieldsList;
        end;
end;

{ Retrieve data on Fields and fill the listbox with it }
procedure TSqlitePassLocateDlg.RefreshFieldsList;
var
i: integer;
begin
{ We fill the listbox with Fields names }
For i := 0 to Pred(MyDataset.Fields.count) do
ListBoxFields.items.Add(MyDataset.Fields[i].FieldName);
end;

procedure TSqlitePassLocateDlg.SbOkClick(Sender: TObject);
var
LocateOptions: TLocateOptions;
begin
LocateOptions := [];
if CbPartialKeys.Checked
   then Include(LocateOptions, loPartialKey);
if CbCaseInsensitive.Checked
   then Include(LocateOptions, loCaseInsensitive);

MyDataset.Locate(TEditKeyFields.Text, VarArrayFromStr(TEditKeyValues.Text), LocateOptions);
ModalResult := mrOk;
end;

procedure TSqlitePassLocateDlg.SbCancelClick(
  Sender: TObject);
begin
ModalResult := mrCancel;
end;


procedure TSqlitePassLocateDlg.ListBoxFieldsDblClick(Sender: TObject);
var
KeyFields, SelectedField: String;
begin
SelectedField := ListBoxFields.Items[ListBoxFields.ItemIndex];
if CheckBoxUseQuotes.Checked
   then MyDataset.SQLSelectStmt.QuoteString(SelectedField);

if TEditKeyFields.Text <> ''
   then KeyFields := TEditKeyFields.Text + ';';

KeyFields := KeyFields + SelectedField;
TEditKeyFields.Text := KeyFields;
TEditKeyFields.SetFocus;
end;

{ Used to Extract values from Dot CSV string }

function TSqlitePassLocateDlg.ExtractStrValue(const Str: string; var Pos: Integer): string;
var
  i: Integer;
begin
  i := Pos;
  while (i <= Length(Str)) and (Str[i] <> ';')
     do Inc(i);
  Result := Copy(Str, Pos, i - Pos);
  if (i <= Length(Str)) and (Str[i] = ';')
     then Inc(i);
  Pos := i;
end;

function TSqlitePassLocateDlg.VarArrayFromStr(LocateStrValues: String): Variant;
var
  i,Pos: Integer;
  Values: TStringList;
begin
  Values := TStringList.Create;
  Result := '';
  Try
  Pos := 1;
  while Pos <= Length(LocateStrValues) do
      Values.Add(ExtractStrValue(LocateStrValues, Pos));

  { Creates the VarArray from TStringList }
  if Values.Count = 1
     then Result := Values[0]
     else begin
          Result := VarArrayCreate([0, Values.Count-1], varVariant);
          for i:=0 to Pred(Values.Count)
              do Result[i] := Values[i];
          end;
  finally
  Values.Free;
  end;

end;

initialization
 {$IFDEF FPC}
  {$I SqlitePassLocateDialog.lrs}
 {$ENDIF}
end.
