{ This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  ---------------------------------------------------------------------------

    Author : Luc DAVID Email: luckylazarus@free.fr
    Last update : 2008-03-31

  --------------------------------------------------------------------------- }

unit SqlitePassVisualTools;
{$IFDEF FPC}
 {$mode DELPHI}
{$ENDIF}

interface

uses
 {$IFDEF FPC}
  LResources,
  LMessages,
  LCLType,
 {$ENDIF}
 {$IFDEF WIN32}
  Windows,
 {$ENDIF}
  Messages, SysUtils, Classes, Graphics, Controls, Forms, Extctrls, Buttons, Math, TypInfo;

type
  TSoLCtrlGrid = class;

  TSoLCtrlGridState = (cgsBrowse, cgsEdit, cgsInsert, cgsAppend, cgsOrder);
  TSoLCtrlGridNotifyEvent = procedure(CtrlGrid: TSoLCtrlGrid) of object;

  TSoLCtrlGrid = class(TScrollBox)
   private
    FBmpBrowse: TBitmap;
    FBmpEdit: TBitmap;
    FBmpInsert: TBitmap;
    FBof: Boolean;
    FControlIndex: Integer;
    FColorUnSelected: TColor;
    FColorSelected: TColor;
    FEof: Boolean;
    FImageEditState: TImage;
    FPanel: TPanel;
    FTabControlNames: TStringList;
    FShowEditState: Boolean;
    FShowDragGrip: Boolean;
    FCanDelete: Boolean;
    FCanInsert: Boolean;
    FActiveRowIndex: Integer;
    FActiveRow: TPanel;
    FRows: TList;
    FOnNewRecord: TSoLCtrlGridNotifyEvent;
    FState: TSoLCtrlGridState;
    FAfterScroll: TSoLCtrlGridNotifyEvent;
    FBeforeScroll: TSoLCtrlGridNotifyEvent;
    procedure SetFPanel(const Value: TPanel);
    function CloneControl(Control: TControl; Parent: TWinControl): TControl;
    Procedure CloneChildControls(Source, Dest: TPanel);
    procedure SetFActiveRowIndex(const Value: Integer);
    procedure SetFActiveRow(Const Value: TPanel);
    function CreateNewRow(PriorRow: Integer = -1): TPanel;
    procedure _OnRowClick(Sender: TObject);
    procedure _OnClick(Sender: TObject);
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure DispachReturnKey;
    procedure SetFState(const Value: TSoLCtrlGridState);
    function GetFRowCount: Integer;
    function GetFIsEmpty: Boolean;
   protected
   public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
   function Append: TPanel;
   function Delete: TPanel;
   function Insert: TPanel;
   procedure Clear;
   procedure First;
   procedure Prior;
   procedure Next;
   procedure Last;
   procedure MoveUp;
   procedure MoveDown;
   Procedure Post;
   procedure Exchange(Index1, Index2: Integer);
   procedure KeyDown(var Key: Word; Shift: TShiftState); override;
   function ControlByName(ControlName: String): TControl;
   procedure FocusActiveRow(const Value: TObject);
   procedure FocusFirstControl;
   Property BOF: Boolean Read FBof;
   Property CanInsert: Boolean Read FCanInsert Write FCanInsert;
   Property CanDelete: Boolean Read FCanDelete Write FCanDelete;
   Property ColorSelected: TColor Read FColorSelected Write FColorSelected;
   Property EOF: Boolean Read FEof;
   Property IsEmpty: Boolean Read GetFIsEmpty;
   Property Panel: TPanel Read FPanel Write SetFPanel;
   Property ActiveRow: TPanel read FActiveRow write SetFActiveRow;
   Property ActiveRowIndex: Integer Read FActiveRowIndex Write SetFActiveRowIndex;
   Property RowCount: Integer Read GetFRowCount;
   Property ShowEditState: Boolean Read FShowEditState Write FShowEditState;
   Property ShowDragGrip: Boolean Read FShowDragGrip Write FShowDragGrip;
   Property State: TSoLCtrlGridState Read FState Write SetFState;
   property OnNewRecord: TSoLCtrlGridNotifyEvent read FOnNewRecord write FOnNewRecord;
   property BeforeScroll: TSoLCtrlGridNotifyEvent read FBeforeScroll write FBeforeScroll;
   property AfterScroll: TSoLCtrlGridNotifyEvent read FAfterScroll write FAfterScroll;
   end;

  TSoLNavButtons = (NavFirst, NavPrior, NavNext, NavLast, NavInsert, NavAppend, NavDelete, NavMoveUp, NavMoveDown);

  TSoLCtrlGridNavigator = class(TCustomPanel)
  private
    FActiveButton: Integer;
    FButtons: array[TSoLNavButtons] of TSpeedButton;
    FButtonCount: Integer;
    FSize: Integer;
    FCtrlGrid: TSoLCtrlGrid;
    FTimer: TTimer;
    procedure _OnMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X,Y: Integer);
    procedure _OnMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X,Y: Integer);
    procedure _OnButtonClick(Btn: Integer);
    Procedure _RefreshButtonState;
    procedure _SetSize(var Width, Height: Integer);
    procedure TimerEvent(Sender:TObject);
    procedure WMSize(var Msg: TWMSize); message WM_SIZE;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation);override;
  public
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;override;
  published
    Property CtrlGrid: TSoLCtrlGrid read FCtrlGrid write FCtrlGrid;
    property Align;
    Property BevelInner;
    Property BevelOuter;
    Property BevelWidth;
    Property BorderWidth;
    Property BorderStyle;
    Property Color;
    Property Ctl3D;
    Property ShowHint;
    Property Tag;
    Property Visible;
  end;


Const
  SolNavButtonName: array[TSoLNavButtons] of PChar = ('FIRST', 'PRIOR', 'NEXT',
    'LAST', 'INSERT', 'APPEND', 'DELETE', 'UP', 'DOWN');


implementation


{$IFNDEF FPC}
 {$R *.RES}
{$ENDIF}


{ TSoLCtrlGrid }

constructor TSoLCtrlGrid.Create(AOwner: TComponent);
begin
Inherited Create(AOwner);
Parent := TWinControl(AOwner);
FRows := TList.Create;
FBmpBrowse := TBitmap.Create;
FBmpEdit := TBitmap.Create;
FBmpInsert := TBitmap.Create;
{$IFDEF FPC}
FBmpBrowse.LoadFromLazarusResource('CG_BROWSE');
FBmpEdit.LoadFromLazarusResource('CG_EDIT');
FBmpInsert.LoadFromLazarusResource('CG_NEW');
{$ELSE}
FBmpBrowse.LoadFromResourceName(HInstance, 'CG_BROWSE');
FBmpEdit.LoadFromResourceName(HInstance, 'CG_EDIT');
FBmpInsert.LoadFromResourceName(HInstance, 'CG_NEW');
{$ENDIF}
FImageEditState := TImage.Create(Self);
FCanInsert := True;
FCanDelete := True;
FTabControlNames := TStringList.Create;
FBof := True;
FEof := True;
With FImageEditState do
     begin
     Transparent := True;
     AutoSize := True;
     Left := 2;
     State := cgsBrowse;
     end;
FActiveRowIndex := -1;
FActiveRow := nil;
Color := $00E2E2E2;
FColorSelected := clBtnShadow;
FColorUnSelected := cl3DLight;
OnClick := _OnClick;
end;

destructor TSoLCtrlGrid.Destroy;
begin
FBmpBrowse.Free;
FBmpEdit.Free;
FBmpInsert.Free;
FRows.Free;
FTabControlNames.Free;
Inherited Destroy;
end;

function TSoLCtrlGrid.CloneControl(Control: TControl; Parent: TWinControl): TControl;
var
MemoryStream: TMemoryStream;
WasNamed: String;
ControlMethod: TMethod;
PData   : PTypeData;
PListe  : PPropList;
//PropertyName: String;
i, PropCount: integer;
begin
  Result := nil;

  { Copies the properties }
  MemoryStream := TMemoryStream.Create;
  try
  WasNamed := Control.Name;
  Control.Name := '';
  MemoryStream.WriteComponent(Control);
  MemoryStream.Position := 0;
  Result := TControlClass(Control.ClassType).Create(Self);
  Result.Parent := Parent;
  MemoryStream.ReadComponent(Result);
  Result.Name := WasNamed + '_' + IntToStr(Random(MaxInt));

  { Assigns the control event methods to the clone ones }
  PData := GetTypeData(PTypeInfo(Control.ClassInfo));
  PropCount := PData^.PropCount;
  New(PListe);
  GetPropInfos(PTypeInfo(Control.ClassInfo), PListe);
  for i := 0 to Pred(PropCount) do
     if PListe^[I]^.PropType^.Kind = tkMethod then
        begin
          ControlMethod := GetMethodProp(Control, PListe^[I]);
          if Assigned(ControlMethod.Code) then
             begin
//             PropertyName := PListe^[I]^.PropType^.Name;
//             if PropertyName := 'TKeyEvent'
//                then SetMethodProp(Result, PListe^[I], ControlMethod);*)
             SetMethodProp(Result, PListe^[I], ControlMethod);
             end;
        end;
  Dispose(PListe);

  finally
  Control.Name := WasNamed;
  MemoryStream.Free;
  end;
end;

Procedure TSoLCtrlGrid.CloneChildControls(Source, Dest: TPanel);
var
i: integer;
NewControl: TControl;
begin
for i:= 0 to Pred(Source.ControlCount) do
    begin
    NewControl := CloneControl(Source.Controls[i], Dest);
    NewControl.Top := Source.Controls[i].top;
    NewControl.Left := Source.Controls[i].Left;
    end;
end;

function TSoLCtrlGrid.CreateNewRow(PriorRow: Integer = -1): TPanel;
begin
Result := TPanel(CloneControl(FPanel, Self));
With Result do
     begin
     Caption := '';
     Visible := True;
     OnClick := _OnRowClick;
     end;

CloneChildControls(FPanel, Result);

if (PriorRow < 0) or (PriorRow > ControlCount)
   { Append }
   then begin
        Result.SetBounds(1, ControlCount * FPanel.Height + 100, Result.Width, Result.Height);
        FRows.Add(Result);
        end
   { Insert }
   else begin
        Result.SetBounds(1, (PriorRow + 1) * FPanel.Height, Result.Width, Result.Height);
        FRows.Insert(PriorRow, Result);
        end;
end;

function TSoLCtrlGrid.Append: TPanel;
begin
  if Not FCanInsert then Exit;
  State := cgsAppend;
  Result := CreateNewRow;
  ActiveRowIndex := Pred(RowCount);
  FocusFirstControl;
  if Assigned(FOnNewRecord) then FOnNewRecord(Self);
end;

function TSoLCtrlGrid.Insert: TPanel;
begin
 if Not FCanInsert then Exit;
 State := cgsInsert;
 Result := CreateNewRow(FActiveRowIndex);
 ActiveRowIndex := FActiveRowIndex;
 if Assigned(FOnNewRecord) then FOnNewRecord(Self);
 FocusFirstControl;
end;

function TSoLCtrlGrid.Delete: TPanel;
begin
 if FCanDelete and Assigned(FActiveRow) then
    begin
    FImageEditState.Parent := nil;
    FRows.Remove(FActiveRow);
    FActiveRow.Destroy;
    FActiveRow := nil;
    If FRows.Count > 0
       then begin
            FState := cgsOrder;
            if FActiveRowIndex < FRows.Count
               then Inc(FActiveRowIndex);
            Prior;
            end
       else begin
            FActiveRowIndex := -1;
            FBof := True;
            FEof := True;
            end;
    end;
end;

procedure TSoLCtrlGrid.Exchange(Index1, Index2: Integer);
var
i: Integer;
Panel1, Panel2, TempPanel: TPanel;
begin
 TempPanel := TPanel.Create(Self);
 Panel1 := TPanel(FRows[Index1]);
 Panel2 := TPanel(FRows[Index2]);
 While Panel1.ControlCount > 0
       do Panel1.Controls[0].Parent := TempPanel;
 While Panel2.ControlCount > 0
       do Panel2.Controls[0].Parent := Panel1;
 While TempPanel.ControlCount > 0
       do TempPanel.Controls[0].Parent := Panel2;
 TempPanel.Free;
 State := cgsOrder;
end;

{ Navigating }

procedure TSoLCtrlGrid.First;
begin
 ActiveRowIndex := 0;
 FocusFirstControl;
 State := cgsBrowse;
end;

procedure TSoLCtrlGrid.Prior;
begin
 ActiveRowIndex := FActiveRowIndex - 1;
 FocusFirstControl;
 State := cgsBrowse;
end;

procedure TSoLCtrlGrid.Next;
begin
 ActiveRowIndex := FActiveRowIndex + 1;
 FocusFirstControl;
 State := cgsBrowse;
end;

procedure TSoLCtrlGrid.Last;
begin
 ActiveRowIndex := Pred(RowCount);
 FocusFirstControl;
 State := cgsBrowse;
end;

procedure TSoLCtrlGrid.MoveUp;
begin
 if Not BOF then
    begin
    Post;
    Exchange(FActiveRowIndex, Pred(FActiveRowIndex));
    ActiveRowIndex := Pred(FActiveRowIndex);
    FocusFirstControl;
    State := cgsBrowse;
    end;
end;

procedure TSoLCtrlGrid.MoveDown;
begin
 if Not EOF then
    begin
    Post;
    Exchange(FActiveRowIndex, Succ(FActiveRowIndex));
    ActiveRowIndex := Succ(FActiveRowIndex);
    FocusFirstControl;
    State := cgsBrowse;
    end;
end;

procedure TSoLCtrlGrid.SetFActiveRowIndex(const Value: Integer);
begin
if (Value < 0) or (Value >= RowCount) then Exit;
if (FActiveRowIndex <> Value) or (FState = cgsInsert) or (FState = cgsOrder) then
   begin
   FBof := Value = 0;
   FEof := Value = Pred(RowCount);
   if Assigned(FActiveRow)
     then FActiveRow.Color := FColorUnSelected;
   if Assigned(FBeforeScroll) and (FState <> cgsOrder)
     then BeforeScroll(Self);
   FActiveRowIndex := Value;
   FActiveRow := TPanel(FRows[FActiveRowIndex]);
   FActiveRow.Color := FColorSelected;
   FImageEditState.Parent := FActiveRow;
   {$IFNDEF FPC}
   Self.ScrollInView(FActiveRow);
   {$ENDIF}
   if Assigned(FAfterScroll)
     then AfterScroll(Self);
   end;
end;

procedure TSoLCtrlGrid._OnRowClick(Sender: TObject);
begin
 ActiveRowIndex := FRows.IndexOf(TPanel(Sender));
end;

procedure TSoLCtrlGrid.SetFPanel(const Value: TPanel);
var
i: Integer;
begin
  FPanel := Value;
  if FPanel.Parent = Self
     then FPanel.Parent := Self.Parent;

  FPanel.Visible := False;
  FPanel.Align := alTop;
  FColorUnSelected := FPanel.Color;
  FImageEditState.Top := (FPanel.Height - FImageEditState.Height) div 2;
  Clear;
  Append;
end;

procedure TSoLCtrlGrid.Clear;
begin
While FRows.Count > 0
      do Delete;
end;

function TSoLCtrlGrid.ControlByName(ControlName: String): TControl;
var
i: integer;
begin
  FControlIndex := -1;
  Result := nil;
  ControlName := Uppercase(ControlName);
  if Not Assigned(ActiveRow) then Exit;
  for i := 0 to Pred(ActiveRow.ControlCount) do
    if Pos(ControlName, Uppercase(ActiveRow.Controls[i].Name)) <> 0 then
       begin
       Result := ActiveRow.Controls[i];
       FControlIndex := i;
       Exit;
       end;
end;

procedure TSoLCtrlGrid._OnClick(Sender: TObject);
begin
 SetFocus;
end;

procedure TSoLCtrlGrid.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  {$IFDEF WIN32}
  Message.Result := DLGC_WANTALLKEYS or DLGC_WANTARROWS;
  {$ENDIF}
end;

procedure TSoLCtrlGrid.KeyDown(var Key: Word; Shift: TShiftState);
begin
case Key of
    VK_HOME : First;
    VK_UP   : if ((State = cgsInsert) or (State = cgsAppend)) and (FActiveRowIndex > 0)
                 then begin
                      Delete;
                      Key := 0;
                      end
                 else Prior;
    VK_PRIOR: Prior;
    VK_DOWN : Next;
    VK_NEXT : Next;
    VK_END  : Last;
    VK_RETURN : begin
                DispachReturnKey;
                Key := 0;
                end;
    else State := cgsEdit;
    end;
end;

procedure TSoLCtrlGrid.DispachReturnKey;
var
F: TCustomForm;
NextActiveControl: TWinControl;
ActiveControlName: String;
begin
F := GetParentForm(Self);
ActiveControlName := Uppercase(F.ActiveControl.Name);
if Pos(Uppercase(FPanel.Controls[Pred(FPanel.ControlCount)].Name), ActiveControlName) <> 0
   then begin
        if FActiveRowIndex = Pred(FRows.Count)
           then Append
           else Next
        end
   else begin
        ControlByName(ActiveControlName);
        If FControlIndex > -1 then
           begin
           NextActiveControl := TWinControl(ActiveRow.Controls[FControlIndex + 1]);
           if NextActiveControl.CanFocus
              then NextActiveControl.SetFocus;
           end;
        end;
end;

procedure TSoLCtrlGrid.FocusFirstControl;
var
FirstControl: TWinControl;
begin
FirstControl := TWinControl(ControlByName(FPanel.Controls[0].Name));
if Assigned(FirstControl) and FirstControl.Showing and FirstControl.CanFocus
   then FirstControl.SetFocus;
end;

procedure TSoLCtrlGrid.SetFState(const Value: TSoLCtrlGridState);
begin
  FState := Value;
  Case FState of
       cgsBrowse: FImageEditState.Picture.Bitmap := FBmpBrowse;
       cgsInsert..cgsAppend: FImageEditState.Picture.Bitmap := FBmpInsert;
       cgsEdit: FImageEditState.Picture.Bitmap := FBmpEdit;
       end;
end;

function TSoLCtrlGrid.GetFRowCount: Integer;
begin
 Result := FRows.Count;
end;

function TSoLCtrlGrid.GetFIsEmpty: Boolean;
begin
 Result := (FRows.Count = 0);
end;

procedure TSoLCtrlGrid.SetFActiveRow(const Value: TPanel);
var
i: Integer;
begin
 i := FRows.IndexOf(Value);
 if i > -1
    then ActiveRowIndex := i;
end;

procedure TSoLCtrlGrid.FocusActiveRow(const Value: TObject);
begin
 if TControl(Value).Parent is TPanel
    then ActiveRow := TPanel(TControl(Value).Parent);
end;

procedure TSoLCtrlGrid.Post;
begin
if Assigned(BeforeScroll)
   then BeforeScroll(Self);
State := cgsBrowse;
end;

{ TSoLCtrlGridNavigator }

constructor TSoLCtrlGridNavigator.Create(AOwner:TComponent);
var
i: TSolNavButtons;
ResName: String;
begin
inherited Create(AOwner);
Parent := TWinControl(AOwner);
FCtrlGrid := nil;
FTimer := TTimer.Create(Self);
FTimer.Enabled := True;
FTimer.OnTimer := TimerEvent;
FTimer.Interval := 200;
Caption := '';
FActiveButton := 0;
ShowHint := True;
Height := 22;
FButtonCount := Succ(Ord(High(FButtons)));
for i := Low(FButtons) to High(FButtons) do
    begin
    FButtons[i]:=TSpeedButton.Create(Self);
    With FButtons[i] do
         begin
         Parent:=Self;
         Flat := True;
         NumGlyphs := 1;
         Tag:= Succ(Ord(i));
         Caption := '';
         Hint := SolNavButtonName[i];
         NumGlyphs := 2;
         FmtStr(ResName, 'BTN_%s', [SolNavButtonName[i]]);
         {$IFDEF FPC}
         Glyph.LoadFromLazarusResource(ResName);
         {$ELSE}
         Glyph.LoadFromResourceName(HInstance, ResName);
         {$ENDIF}
         OnMouseDown := _OnMouseDown;
         OnMouseUp := _OnMouseUp;
         end;
end;
Width := FButtonCount * 22;
{$IFDEF FPC}
_SetSize(Width, Height);
{$ENDIF}
end;

destructor TSoLCtrlGridNavigator.Destroy;
var
  i: TSoLNavButtons;
begin
  FTimer.Free;
  For i:=Low(FButtons) to High(FButtons) do FButtons[i].Free;
  inherited Destroy;
end;

procedure TSoLCtrlGridNavigator._OnButtonClick(Btn: Integer);
begin
With FCtrlGrid do
   begin
   Case Btn of
   1: First;
   2: Prior;
   3: Next;
   4: Last;
   5: Insert;
   6: Append;
   7: Delete;
   8: MoveUp;
   9: MoveDown;
   end;
   end;
_RefreshButtonState;
end;

procedure TSoLCtrlGridNavigator.TimerEvent;
begin
  { Only for scrolling }
  (*Case FActiveButton of
   2..3: DoButtonClick(FActiveButton);
   end;*)
   _RefreshButtonState;
end;


Procedure TSoLCtrlGridNavigator._RefreshButtonState;
begin
  if csLoading in ComponentState then Exit;
  FButtons[NavFirst].Enabled    := Not CtrlGrid.BOF;
  FButtons[NavPrior].Enabled    := Not CtrlGrid.BOF;
  FButtons[NavMoveUp].Enabled   := Not CtrlGrid.BOF;
  FButtons[NavNext].Enabled     := Not CtrlGrid.EOF;
  FButtons[NavLast].Enabled     := Not CtrlGrid.EOF;
  FButtons[NavMoveDown].Enabled := Not CtrlGrid.EOF;
  FButtons[NavDelete].Enabled   := Not CtrlGrid.IsEmpty;
  FButtons[NavInsert].Enabled   := Not CtrlGrid.IsEmpty;
end;

procedure TSoLCtrlGridNavigator.WMSize(var Msg: TWMSize);
var
  W, H: Integer;
begin
  inherited;
  { check for minimum size }
  W := Width;
  H := Height;
  _SetSize(W, H);
  Inherited Resize;
  Invalidate;
  Msg.Result := 0;
end;

procedure TSoLCtrlGridNavigator._SetSize(var Width, Height: Integer);
var
 i: TSoLNavButtons;
 ButtonHeight, ButtonWidth, BorderWidth: integer;
begin
 if (csLoading in ComponentState)
    then Exit;

 ButtonWidth := Width div FButtonCount;

 { Ajust panel width }
 Width := (ButtonWidth * FButtonCount) + 1;

 BorderWidth := 1;
 If BevelInner <> bvNone
    then Inc(BorderWidth, BevelWidth);
 ButtonHeight := Height - BorderWidth;

 { Left, Top, Width, Height }
 for i := Low(FButtons) to High(FButtons)
     do FButtons[i].SetBounds((Ord(i)* ButtonWidth) + BorderWidth, 1, ButtonWidth, ButtonHeight);
end;

procedure TSoLCtrlGridNavigator._OnMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X,Y: Integer);
begin
  FActiveButton := (Sender as TSpeedButton).Tag;
  _OnButtonClick(FActiveButton);
end;

procedure TSoLCtrlGridNavigator._OnMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState;
    X,Y: Integer);
begin
  FActiveButton := 0;
end;

procedure TSoLCtrlGridNavigator.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FCtrlGrid <> nil) and
    (AComponent = FCtrlGrid) then FCtrlGrid := nil;
end;

initialization
 {$IFDEF FPC}
  {$i SqlitePassVisualTools.lrs}
 {$ENDIF}
 
end.
