
{ This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  ---------------------------------------------------------------------------

    Author : Luc DAVID Email: luckylazarus@free.fr
    2006 - 2008

    Major changes are indicated in the \Documentation\Changes.pdf file
    Last update 31.05.2008

  --------------------------------------------------------------------------- }


{ TSqlitePassIndex }

constructor TSqlitePassIndex.Create;
begin
 inherited create;
 FColumns := TSqlitePassIndexColumns.Create(Self);
end;

destructor TSqlitePassIndex.Destroy;
begin
 FColumns.Free;
 inherited Destroy;
end;

procedure TSqlitePassIndex.SetFIndexName(Value: String);
begin
if IsValidIdent(Value)
   then FIndexName := Value
   else raise Exception.CreateFmt(Msg3220,[Value]);
end;


{ TSqlitePassDatabaseIndex }

constructor TSqlitePassDatabaseIndex.Create(Owner: TSqlitePassDatabaseIndexDefs);
begin
 inherited create;
 FIndexDefs := Owner;
end;

destructor TSqlitePassDatabaseIndex.Destroy;
begin
 FIndexDefs := nil;
 inherited Destroy;
end;

procedure TSqlitePassDatabaseIndex.DeleteIndex;
begin
 FindexDefs.FDatabase.Engine.ExecSQL('DROP INDEX ' + FIndexName);
end;

function TSqlitePassDatabaseIndex.GetAttached: Boolean;
begin
 Result := FIndexDefs.Attached;
end;

procedure TSqlitePassDatabaseIndex.Reindex;
begin
 FIndexDefs.FDatabase.Engine.ExecSQL('REINDEX ' + FIndexName);
end;


{ TSqlitePassDatabaseIndexDefs }

constructor TSqlitePassDatabaseIndexDefs.Create(Owner: TSqlitePassDatabase; AttachedDb: TSqlitePassDatabaseAttached = nil);
begin
inherited create;
FDatabase := Owner;
FAttachedDatabase := AttachedDb;
end;

destructor TSqlitePassDatabaseIndexDefs.Destroy;
begin
ClearAndFreeItems;
FAttachedDatabase := nil;
FDatabase := nil;
inherited Destroy;
end;

function TSqlitePassDatabaseIndexDefs.GetItem(Index: Integer): TSqlitePassIndex;
begin
Result:=TSqlitePassIndex(Inherited Items[Index]);
end;

procedure TSqlitePassDatabaseIndexDefs.SetItem(Index: Integer;
  const Value: TSqlitePassIndex);
begin
  Put(Index,Value);
end;

procedure TSqlitePassDatabaseIndexDefs.ClearAndFreeItems;
var
i: Integer;
begin
{ The Datasets.IndexDefs points to DatabaseIndexDefs, so we clear it first }
// TODO - Verify if necessary -> Raise Exception
(*if Assigned(FDatabase.Datasets) then
   begin
   for i := 0 to Pred(FDatabase.Datasets.Count)
       do FDatabase.Datasets[i].IndexDefs.Clear;
   end; *)
{ Clear the internal items }
for i := 0 to Pred(Count) do
    Items[i].Free;
{ Clear the list pointers }
inherited Clear;
end;

procedure TSqlitePassDatabaseIndexDefs.CreateIndex(Sql: String);
begin
 FDatabase.Engine.ExecSQL(Sql);
 // TODO - Optimize -> don't refresh ALL indexes info but just add only the new one
 Refresh; 
end;

procedure TSqlitePassDatabaseIndexDefs.CreateIndex(Index: TSqlitePassDatasetIndex);
var
i: integer;
SqlStmt, UniqueStmt, FieldStmt: String;
begin
If Index.Unique
   then UniqueStmt := 'UNIQUE';

SqlStmt :=  'CREATE ' + UniqueStmt + ' INDEX "' + Index.IndexName +
            '" ON "' + Index.TableName + '" (';

For i := 0 to Pred(Index.Columns.Count) do
    begin
    FieldStmt := '"'+Index.Columns[i].ColumnName+'"';
    if Index.Columns[i].Direction = cidAscending
       then FieldStmt := FieldStmt + ' ASC, '
       else FieldStmt := FieldStmt + ' DESC, ';
    SqlStmt := SqlStmt + FieldStmt;
    end;
System.Delete(SqlStmt, Length(SqlStmt)-1,1);
SqlStmt := SqlStmt + ');';
FDatabase.Engine.ExecSQL(SqlStmt);
// TODO - Optimize -> don't refresh ALL indexes info but just add only the new one
Refresh;
end;

procedure TSqlitePassDatabaseIndexDefs.DeleteIndex(IndexName: String);
begin
 FDatabase.Engine.ExecSQL('DROP INDEX ' + IndexName + ';');
 // TODO - Optimize -> don't refresh ALL indexes info but just delete only the new one
 Refresh;
end;

function TSqlitePassDatabaseIndexDefs.FindIndex(
  const Value: string): TSqlitePassIndex;
var
  I: Integer;
begin
  for I := 0 to Pred(Count) do
  begin
    Result := TSqlitePassIndex(inherited Items[I]);
    if AnsiCompareText(Result.IndexName, Value) = 0 then Exit;
  end;
  Result := nil;
end;

function TSqlitePassDatabaseIndexDefs.GetAttached: Boolean;
begin
 Result := (FAttachedDatabase <> nil);
end;

function TSqlitePassDatabaseIndexDefs.IndexByName(
  const Value: string): TSqlitePassIndex;
begin
  Result := FindIndex(Value);
  if Result = nil then
    DatabaseErrorFmt(Msg3200, [Value], FDatabase);
end;

procedure TSqlitePassDatabaseIndexDefs.Refresh;
var
i: Integer;
begin
 FDatabase.Translator.GetDatabaseIndexDefs(Self);
 For i := 0 to Pred(FDatabase.Datasets.Count)
     do FDatabase.Datasets[i].IndexDefs.RefreshFromCache;
end;

procedure TSqlitePassDatabaseIndexDefs.Reindex;
begin
// TODO - TSqlitePassDatabaseIndexDefs.Reindex - Optimise
 FDatabase.Engine.ExecSQL('Reindex Binary');
end;

procedure TSqlitePassDatabaseIndexDefs.RenameIndex(OldIndexName,
  NewIndexName: String);
var
CurrentIndex: TSqlitePassIndex;
begin
if NewIndexName = ''
   then exit;
CurrentIndex := IndexByName(OldIndexName);
If Assigned(CurrentIndex)
   then begin
        CurrentIndex.IndexName := NewIndexName;
        CreateIndex(TSqlitePassDatasetIndex(CurrentIndex));
        DeleteIndex(OldIndexName);
        end;
end;



{ TSqlitePassTableDef }

constructor TSqlitePassTableDef.Create(Owner: TSqlitePassTableDefs);
begin
  Inherited Create;
  FPrimaryKeyField := nil;
  FTableDefs := Owner;
  FFieldDefs := TSqlitePassTableFieldDefs.Create(Self);
end;

destructor TSqlitePassTableDef.Destroy;
begin
  FFieldDefs.Free;
  FPrimaryKeyField := nil;
  FTableDefs := nil;
  Inherited Destroy;
end;

procedure TSqlitePassTableDef.DeleteTable(Silent: Boolean = False);
begin
 FTableDefs.DeleteTable(FTableName, Silent);
end;

procedure TSqlitePassTableDef.Reindex;
begin
 FTableDefs.FDatabase.Engine.ExecSQL('Reindex ' + FTableName);
end;

procedure TSqlitePassTableDef.RenameAs(NewName: String);
begin
 // TODO - Change Associated triggers to new table name
 if NewName <> ''
    then FTableDefs.FDatabase.Engine.ExecSQL('ALTER TABLE ' + FTableName + ' RENAME TO ' + NewName);
end;


{ TSqlitePassTableDefs }

constructor TSqlitePassTableDefs.Create(Owner: TSqlitePassDatabase; AttachedDb: TSqlitePassDatabaseAttached = nil);
begin
inherited Create;
FDatabase := Owner;
FAttachedDatabase := AttachedDb;
end;

destructor TSqlitePassTableDefs.Destroy;
begin
ClearAndFreeItems;
FAttachedDatabase := nil;
FDatabase := nil;
inherited Destroy;
end;

procedure TSqlitePassTableDefs.ClearAndFreeItems;
var
i: Integer;
begin
{ Clear the internal items }
for i := 0 to Pred(Count)
   do if Assigned(Items[i]) then Items[i].Free;
{ Clear the list pointers }
inherited Clear;
end;

procedure TSqlitePassTableDefs.CreateTable(Sql: String);
begin
 FDatabase.Engine.ExecSQL(Sql);
end;

procedure TSqlitePassTableDefs.DeleteTable(Const TableName: String; Silent: Boolean = False);
var
SilentMode: String;
begin
 If Silent
    then SilentMode := 'if exists '
    else SilentMode := '';

 FDatabase.Engine.ExecSQL('Drop table ' + SilentMode + TableName + ';');
 { Gets Tables Definitions from database }
 FDatabase.Translator.GetDatabaseTableDefs(Self);
end;

procedure TSqlitePassTableDefs.EmptyTable(Const TableName: String);
begin
 FDatabase.Engine.ExecSQL('DELETE FROM "' + TableName + '";');
end;

function TSqlitePassTableDefs.FindTable(const Value: string): TSqlitePassTableDef;
var
  I: Integer;
begin
  for I := 0 to Pred(Count) do
  begin
    Result := TSqlitePassTableDef(inherited Items[I]);
    if AnsiCompareText(Result.TableFullName, Value) = 0 then Exit;
  end;
  Result := nil;
end;

function TSqlitePassTableDefs.GetAttached: Boolean;
begin
 Result := (FAttachedDatabase <> nil);
end;

function TSqlitePassTableDefs.GetItem(Index: Integer): TSqlitePassTableDef;
begin
Result:=TSqlitePassTableDef(Inherited Items[Index]);
end;

procedure TSqlitePassTableDefs.Refresh;
begin
{ Loads the DatatypeOptions if needed }
If Not (soManual in FDatabase.FDataTypeOptions.LoadOptions)
   then FDatabase.FDatatypeOptions.LoadFromDatabase;

{ Finally loads default TranslationRules if none are already defined }
if (FDatabase.FDatatypeOptions.TranslationRules.Count = 0)
   then FDatabase.FDatatypeOptions.SetDefaultValues(FDatabase.FDatabaseType, True);

{ Gets Tables Definitions from database }
FDatabase.Translator.GetDatabaseTableDefs(Self);

{ Overrides fieldDefs settings with CustomFieldDefs Settings }
FDatabase.DataTypeOptions.ApplyCustomFieldDefs;
end;

procedure TSqlitePassTableDefs.SetItem(Index: Integer;
  const Value: TSqlitePassTableDef);
begin
  Put(Index,Value);
end;

function TSqlitePassTableDefs.TableByName(
  const Value: string): TSqlitePassTableDef;
begin
  Result := FindTable(Value);
  if Result = nil then
    DatabaseErrorFmt(Msg3700, [Value], FDatabase);
end;

function TSqlitePassTableDefs.FindFieldDef(TableName, FieldName: String): TSqlitePassTableFieldDef;
var
TableDef: TSqlitePassTableDef;
begin
 Result := nil;
 TableDef := FindTable(TableName);
 if TableDef <> nil
    then Result := TableDef.FieldDefs.FindFieldDef(FieldName);
end;

function TSqlitePassTableDefs.FindFieldTableName(FieldName: String; TableNames: TStringList): String;
var
i: integer;
FieldDef: TSqlitePassTableFieldDef;
begin
{ If TableNames is empty, we scan ALL the TableDefs.FieldDefs }
if TableNames.Count = 0 then
   For i := 0 to Pred(Count)
       do TableNames.Add(Items[i].TableName);

Result := '';
for i := 0 to Pred(TableNames.Count) do
  begin
  FieldDef := FindFieldDef(TableNames[i], FieldName);
  if FieldDef <> nil then
     begin
     Result := TableNames[i];
     Exit;
     end;
  end;
end;


{ TSqlitePassView }

constructor TSqlitePassView.Create(Owner: TSqlitePassViews);
begin
 inherited Create;
 FViews := Owner;
end;

destructor TSqlitePassView.Destroy;
begin
 FViews := nil;
 inherited Destroy;
end;

procedure TSqlitePassView.DeleteView;
begin
 FViews.DeleteView(FViewName);
end;

function TSqlitePassView.GetAttached: Boolean;
begin
 Result := FViews.Attached;
end;


{ TSqlitePassViews }

constructor TSqlitePassViews.Create(Owner: TSqlitePassDatabase; AttachedDb: TSqlitePassDatabaseAttached = nil);
begin
 inherited Create;
 FDatabase := Owner;
 FAttachedDatabase := AttachedDb;
end;

destructor TSqlitePassViews.Destroy;
begin
ClearAndFreeItems;
FAttachedDatabase := nil;
FDatabase := nil;
inherited Destroy;
end;

procedure TSqlitePassViews.ClearAndFreeItems;
var
i: Integer;
begin
{ Clear the internal items }
for i := 0 to Pred(Count)
    do Items[i].Free;
{ Clear the list pointers }
inherited Clear;
end;

procedure TSqlitePassViews.CreateView(Const ViewName, SQL: String;
          Const Temp: Boolean = False; Const Silent: Boolean = True);
var
TempView, SilentMode: String;
begin
If Temp
   then TempView := ' TEMPORARY '
   else TempView := '';

If Silent
  then SilentMode := 'IF NOT EXISTS '
  else SilentMode := '';

 FDatabase.Engine.ExecSQL('CREATE VIEW ' + TempView + SilentMode + ' "' + ViewName + '" AS ' + SQL + ';');
 Refresh;
end;

procedure TSqlitePassViews.DeleteView(ViewName: String);
begin
 FDatabase.Engine.ExecSQL('DROP VIEW ' + ViewName + ';');
 Refresh;
end;

function TSqlitePassViews.FindView(const Value: string): TSqlitePassView;
var
  I: Integer;
begin
  for I := 0 to Pred(Count) do
  begin
    Result := TSqlitePassView(inherited Items[I]);
    if AnsiCompareText(Result.ViewName, Value) = 0 then Exit;
  end;
  Result := nil;
end;

function TSqlitePassViews.GetAttached: Boolean;
begin
 Result := (FAttachedDatabase <> nil);
end;

function TSqlitePassViews.GetItem(Index: Integer): TSqlitePassView;
begin
 Result:=TSqlitePassView(Inherited Items[Index]);
end;

procedure TSqlitePassViews.Refresh;
begin
 FDatabase.Translator.GetDatabaseViews(Self);
end;

procedure TSqlitePassViews.SetItem(Index: Integer;
  const Value: TSqlitePassView);
begin
  Put(Index,Value);
end;

function TSqlitePassViews.ViewByName(const Value: string): TSqlitePassView;
begin
  Result := FindView(Value);
  if Result = nil
     then DatabaseErrorFmt(Msg3400, [Value], FDatabase);
end;


{ TSqlitePassTrigger }

constructor TSqlitePassTrigger.Create(Owner: TSqlitePassTriggers);
begin
 inherited Create;
 FTriggers := Owner;
end;

destructor TSqlitePassTrigger.Destroy;
begin
 FTriggers := nil;
 inherited Destroy;
end;

procedure TSqlitePassTrigger.DeleteTrigger;
begin
 FTriggers.FDatabase.Engine.ExecSQL('Drop trigger '+ FTriggerName + ';');
end;

function TSqlitePassTrigger.GetAttached: Boolean;
begin
 Result := FTriggers.Attached;
end;


{ TSqlitePassTriggers }

constructor TSqlitePassTriggers.Create(Owner: TSqlitePassDatabase; AttachedDb: TSqlitePassDatabaseAttached = nil);
begin
 inherited Create;
 FDatabase := Owner;
 FAttachedDatabase := AttachedDb;
end;

destructor TSqlitePassTriggers.Destroy;
begin
ClearAndFreeItems;
FAttachedDatabase := nil;
FDatabase := nil;
inherited Destroy;
end;

procedure TSqlitePassTriggers.ClearAndFreeItems;
var
i: Integer;
begin
{ Clear the internal items }
for i := 0 to Pred(Count)
    do Items[i].Free;
{ Clear the list pointers }
inherited Clear;
end;

procedure TSqlitePassTriggers.CreateTrigger(Sql: String);
begin
 FDatabase.Engine.ExecSQL(Sql);
end;

procedure TSqlitePassTriggers.DeleteTrigger(TriggerName: String);
begin
  FDatabase.Engine.ExecSQL('Drop trigger ' + TriggerName + ';');
end;

function TSqlitePassTriggers.FindTrigger(
  const Value: string): TSqlitePassTrigger;
var
  I: Integer;
begin
  for I := 0 to Pred(Count) do
  begin
    Result := TSqlitePassTrigger(inherited Items[I]);
    if AnsiCompareText(Result.TriggerName, Value) = 0 then Exit;
  end;
  Result := nil;
end;

function TSqlitePassTriggers.GetAttached: Boolean;
begin
 Result := (FAttachedDatabase <> nil);
end;

function TSqlitePassTriggers.GetItem(Index: Integer): TSqlitePassTrigger;
begin
Result:=TSqlitePassTrigger(Inherited Items[Index]);
end;

procedure TSqlitePassTriggers.Refresh;
begin
 FDatabase.Translator.GetDatabaseTriggers(Self);
end;

procedure TSqlitePassTriggers.SetItem(Index: Integer;
  Value: TSqlitePassTrigger);
begin
   Put(Index,Value);
end;

function TSqlitePassTriggers.TriggerByName(
  const Value: string): TSqlitePassTrigger;
begin
  Result := FindTrigger(Value);
  if Result = nil then
    DatabaseErrorFmt(Msg3500, [Value], FDatabase);
end;


{ TSqlitePassQueryDef }

constructor TSqlitePassQueryDef.Create(Owner: TSqlitePassQueryDefs);
begin
inherited Create;
FQueryDefs:=Owner;
FFieldDefs := TFieldDefs.Create(nil);
end;

destructor TSqlitePassQueryDef.Destroy;
begin
FFieldDefs.Free;
FQueryDefs:=nil;
inherited Destroy;
end;

procedure TSqlitePassQueryDef.DeleteQuery;
begin
// TODO : TSqlitePassQueryDef.DeleteQuery - To be implemented
end;

function TSqlitePassQueryDef.GetAttached: Boolean;
begin
 Result := FQueryDefs.Attached;
end;

procedure TSqlitePassQueryDef.RenameAs(NewName: String);
begin
// TODO : TSqlitePassQueryDef.RenameAs - To be implemented
end;


{ TSqlitePassQueryDefs }

constructor TSqlitePassQueryDefs.Create(Owner: TSqlitePassDatabase; AttachedDb: TSqlitePassDatabaseAttached = nil);
begin
inherited Create;
FDatabase := Owner;
FAttachedDatabase := nil;
end;

destructor TSqlitePassQueryDefs.Destroy;
begin
ClearAndFreeItems;
FAttachedDatabase := nil;
FDatabase := nil;
inherited Destroy;
end;

procedure TSqlitePassQueryDefs.ClearAndFreeItems;
var
i: integer;
begin
{ Clears the internal items }
For i := 0 to Pred(Count)
    do Items[i].Free;
{ Clears the list pointers }
inherited Clear;
end;

procedure TSqlitePassQueryDefs.CreateQuery(Sql: String);
begin
// TODO : TSqlitePassQueryDefs.CreateQuery - To be implemented
end;

procedure TSqlitePassQueryDefs.DeleteQuery(QueryName: String);
begin
// TODO : TSqlitePassQueryDefs.DeleteQuery - To be implemented
end;

function TSqlitePassQueryDefs.FindQuery(
  const Value: string): TSqlitePassQueryDef;
var
  I: Integer;
begin
  for I := 0 to Pred(Count) do
  begin
    Result := TSqlitePassQueryDef(inherited Items[I]);
    if AnsiCompareText(Result.QueryFullName, Value) = 0 then Exit;
  end;
  Result := nil;
end;

function TSqlitePassQueryDefs.GetAttached: Boolean;
begin
 Result := (FAttachedDatabase <> nil);
end;

function TSqlitePassQueryDefs.GetItem(Index: Integer): TSqlitePassQueryDef;
begin
 Result:=TSqlitePassQueryDef(Inherited Items[Index]);
end;

procedure TSqlitePassQueryDefs.SetItem(Index: Integer;
  const Value: TSqlitePassQueryDef);
begin
  Put(Index,Value);
end;

function TSqlitePassQueryDefs.QueryByName(
  const Value: string): TSqlitePassQueryDef;
begin
  Result := FindQuery(Value);
  if Result = nil then
    DatabaseErrorFmt(Msg3300, [Value], FDatabase);
end;

procedure TSqlitePassQueryDefs.Refresh;
begin
 FDatabase.Translator.GetDatabaseQueryDefs(Self);
end;

{ TSqlitePassIndexColumns }

constructor TSqlitePassIndexColumns.Create(Owner: TSqlitePassIndex);
begin
inherited Create;
FIndex := Owner;
end;

destructor TSqlitePassIndexColumns.Destroy;
begin
ClearAndFreeItems;
FIndex := nil;
inherited Destroy;
end;

procedure TSqlitePassIndexColumns.ClearAndFreeItems;
var
i: Integer;
begin
{ Clears the internal items }
for i := 0 to Pred(Count)
    do Items[i].Free;
{ Clears the list pointers }
inherited Clear;
end;

function TSqlitePassIndexColumns.ColumnByName(
  const Value: string): TSqlitePassIndexColumn;
begin
  Result := FindColumn(Value);
  if Result = nil
     then DatabaseErrorFmt(Msg3200, [Value], nil);
end;

function TSqlitePassIndexColumns.FindColumn(
  const Value: string): TSqlitePassIndexColumn;
var
  I: Integer;
begin
  for I := 0 to Pred(Count) do
  begin
    Result := TSqlitePassIndexColumn(inherited Items[I]);
    if AnsiCompareText(Result.ColumnName, Value) = 0 then Exit;
  end;
  Result := nil;
end;

function TSqlitePassIndexColumns.GetItem(
  Index: Integer): TSqlitePassIndexColumn;
begin
  Result:=TSqlitePassIndexColumn(Inherited Items[Index]);
end;

procedure TSqlitePassIndexColumns.SetItem(Index: Integer;
  const Value: TSqlitePassIndexColumn);
begin
  Put(Index,Value);
end;


{ TSqlitePassDatabasesAttached }

constructor TSqlitePassDatabasesAttached.Create(Owner: TSqlitePassDatabase);
begin
 Inherited Create;
 FDatabase := Owner;
end;

Destructor TSqlitePassDatabasesAttached.Destroy;
begin
 DetachAll;
 FDatabase := nil;
 Inherited Destroy;
end;

procedure TSqlitePassDatabasesAttached.DetachAll;
begin
While Count > 0 do FDatabase.DetachDatabase(GetItem(Pred(Count)).AliasName)
end;

function TSqlitePassDatabasesAttached.DatabaseByName(
  const Name: String): TSqlitePassDatabaseAttached;
begin
  Result := FindDatabase(Name);
  if Result = nil
     then DatabaseErrorFmt(Msg1800, [Name], FDatabase);
end;

function TSqlitePassDatabasesAttached.FindDatabase(
  const Value: string): TSqlitePassDatabaseAttached;
var
  I: Integer;
begin
  for I := 0 to Pred(Count) do
  begin
    Result := TSqlitePassDatabaseAttached(inherited Items[I]);
    if AnsiCompareText(Result.AliasName, Value) = 0 then Exit;
  end;
  Result := nil;
end;

function TSqlitePassDatabasesAttached.GetItem(
  Index: Integer): TSqlitePassDatabaseAttached;
begin
 Result:=TSqlitePassDatabaseAttached(Inherited Items[Index]);
end;

procedure TSqlitePassDatabasesAttached.SetItem(Index: Integer;
  Database: TSqlitePassDatabaseAttached);
begin
 Put(Index,Database);
end;

{ TSqlitePassDatabaseAttached }

constructor TSqlitePassDatabaseAttached.Create(Databases: TSqlitePassDatabasesAttached);
begin
  Inherited Create;
  FEngine := TSqlitePassEngine.Create(nil);
  FDatabasesList := Databases;
end;

destructor TSqlitePassDatabaseAttached.Destroy;
begin
  FDatabasesList := nil;
  FEngine.Free;
  Inherited Destroy;
end;

{ TSqlitePassTableFieldDef }

constructor TSqlitePassTableFieldDef.Create(
  Owner: TSqlitePassTableFieldDefs);
begin
 Inherited Create;
 FFieldDefs := Owner;
end;

destructor TSqlitePassTableFieldDef.Destroy;
begin
 FFieldDefs := nil;
 Inherited Destroy;
end;

function TSqlitePassTableFieldDef.GetFFieldFullName: String;
begin
 Result := FFieldDefs.FTableDef.TableName + '.' + FFieldName;
end;


{ TSqlitePassTableFieldDefs }

constructor TSqlitePassTableFieldDefs.Create(Owner: TSqlitePassTableDef);
begin
 Inherited Create;
 FTableDef := Owner;
end;

destructor TSqlitePassTableFieldDefs.Destroy;
begin
 ClearAndFreeItems;
 FTableDef := nil;
 Inherited Destroy;
end;

function TSqlitePassTableFieldDefs.FieldDefByName(
  const Value: string): TSqlitePassTableFieldDef;
begin
  Result := FindFieldDef(Value);
  if Result = nil then
    DatabaseErrorFmt(Msg3600, [Value], nil);
end;

function TSqlitePassTableFieldDefs.FindFieldDef(
  const Value: string): TSqlitePassTableFieldDef;
var
  i: Integer;
begin
  Result := nil;
  if (AnsiCompareText(Value,'Rowid') = 0)
  or (AnsiCompareText(Value,'Oid') = 0)
  or (AnsiCompareText(Value,'_Rowid_') = 0)
     then Result := FTableDef.FPrimaryKeyField
     else begin
          for i := 0 to Pred(Count) do
              begin
              Result := TSqlitePassTableFieldDef(inherited Items[I]);
              if AnsiCompareText(Result.FieldName, Value) = 0 then Exit;
              end;
          end;    
end;

function TSqlitePassTableFieldDefs.GetItem(Index: Integer): TSqlitePassTableFieldDef;
begin
  Result:=TSqlitePassTableFieldDef(Inherited Items[Index]);
end;

procedure TSqlitePassTableFieldDefs.SetItem(Index: Integer;
  const Value: TSqlitePassTableFieldDef);
begin
  Put(Index,Value);
end;

procedure TSqlitePassTableFieldDefs.ClearAndFreeItems;
var
i: Integer;
begin
{ Clears the internal items }
for i := 0 to Pred(Count)
    do Items[i].Free;
{ Clears the list pointers }
inherited Clear;
end;


{ TSqlitePassDatasets }

constructor TSqlitePassDatasets.Create(Owner: TSqlitePassDatabase);
begin
inherited Create;
FDatabase := Owner;
end;

destructor TSqlitePassDatasets.Destroy;
begin
FDatabase := nil;
inherited Destroy;
end;

function TSqlitePassDatasets.FindDataset(
  const Value: string): TSqlitePassDataset;
var
  I: Integer;
begin
for I := 0 to Pred(Count) do
  begin
    Result := TSqlitePassDataset(inherited Items[I]);
    if AnsiCompareText(Result.FDatasetFullName, Value) = 0 then Exit;
  end;
  Result := nil;
end;

Function TSqlitePassDatasets.GetItem(Index: Integer): TSqlitePassDataset;
begin
 Result:=TSqlitePassDataset(Inherited Items[Index]);
end;

Procedure TSqlitePassDatasets.SetItem(Index: Integer; Dataset: TSqlitePassDataset);
begin
Put(Index,Dataset);
end;

function TSqlitePassDatasets.DatasetByName(Const Name: String): TSqlitePassDataset;
var
Index: Integer;
begin
Result := Nil;
for index := 0 to Count -1 do
    if Items[Index].Name = Name
       then begin
            result := Items[Index];
            Exit
            end;
If Result = nil then raise Exception.CreateFmt(Msg2070,[Name])
end;

{ Closes all opened datasets }
procedure TSqlitePassDatasets.Close;
var
i: Integer;
begin
For i := 0 to Pred(Count)
    do Items[i].Close;
end;



{ TSqlitePassSQLStmts }

constructor TSqlitePassSqlStmts.Create(Owner: TSqlitePassDatabase; AttachedDb: TSqlitePassDatabaseAttached = nil);
begin
inherited Create;
FDatabase := Owner;
FAttachedDatabase := AttachedDb;
end;

destructor TSqlitePassSqlStmts.Destroy;
begin
ClearAndFreeItems;
FAttachedDatabase := nil;
FDatabase := nil;
inherited Destroy;
end;

procedure TSqlitePassSqlStmts.ClearAndFreeItems;
var
i: Integer;
begin
{ Clear the internal items }
for i := 0 to Pred(Count)
   do if Assigned(Items[i]) then Items[i].Free;
{ Clear the list pointers }
inherited Clear;
end;

procedure TSqlitePassSqlStmts.UpdateSQLStmtTable(Const UpdateType: Integer; Const SQLStmtName: String;
                                                 Const NewSQLStmtName: String; Const SQLStmt: TStringList);
Const
Sql = 'CREATE TABLE IF NOT EXISTS "' + SqlitePassSQLStmtsTable + '" ("Name" TEXT(100) NOT NULL UNIQUE, "Statement" CLOB);';
var
WasConnected, WasReadOnly: Boolean;
NewSQLStmt: TSqlitePassSQLStmt;
DbSQLStmtsDataset: TSqlitePassDataset;
DbSQLStmtsTableDef: TSqlitePassTableDef;
begin
If Assigned(FDatabase) and Assigned(FDatabase.FEngine) then
   begin
   try
     DbSQLStmtsDataset := nil;
     WasConnected := FDatabase.Connected;
     WasReadOnly  := FDatabase.ReadOnly;
     FDatabase.ReadOnly := False;
     FDatabase.Open;

     { Creates the table if it doesn't already exist }
     DbSQLStmtsTableDef := FDatabase.FTableDefs.FindTable(SqlitePassSQLStmtsTable);
     if DbSQLStmtsTableDef = nil then
        begin
        FDatabase.FEngine.ExecSQL(Sql);
        FDatabase.Translator.GetDatabaseTableDefs(FDatabase.TableDefs);
        DbSQLStmtsTableDef := FDatabase.FTableDefs.FindTable(SqlitePassSQLStmtsTable)
        end;

     { Enforces fields fieldTypes to ftMemo whatever Databasetype is used }
      DbSQLStmtsTableDef.FieldDefs[0].DataType := ftString;
      DbSQLStmtsTableDef.FieldDefs[1].DataType := ftMemo;

     { Saves SQLStmts to database }
     DbSQLStmtsDataset:= TSqlitePassDataset.Create(nil);
     DbSQLStmtsDataset.Database := FDatabase; { Kept before With .. do because of incorrect compiler assigment ? }
     With DbSQLStmtsDataset do
          begin
          DatasetName := SqlitePassSQLStmtsTable;
          ReadOnly := False;
          Open;
          Case UpdateType of
          { Create or Update }
          0: begin
             if Locate('Name', SQLStmtName, [loCaseInsensitive])
                then begin
                     Edit;
                     SQLStmtByName(SQLStmtName).SQL.Assign(SQLStmt);

                     end
                else begin
                     Append;
                     NewSQLStmt := TSqlitePassSQLStmt.Create(Self);
                     NewSQLStmt.StmtName := SQLStmtName;
                     NewSQLStmt.SQL.Assign(SQLStmt);
                     Add(NewSQLStmt);
                     end;
              FieldByName('Name').Value := SQLStmtName;
              FieldByName('Statement').Assign(SQLStmt);
              Post;
              end;
          { Rename }
          1: begin
             if Locate('Name', SQLStmtName, [loCaseInsensitive])
                then begin
                     Edit;
                     FieldByName('Name').Value := NewSQLStmtName;
                     Post;
                     SQLStmtByName(SQLStmtName).StmtName := NewSQLStmtName;
                     end;
             end;

          { Delete }
          2: begin
             if Locate('Name', SQLStmtName, [loCaseInsensitive])
                then begin
                     Delete;
                     Remove(SQLStmtByName(SQLStmtName));
                     end;
             end;
          end;
      end { With }
   finally
     if Assigned(DbSQLStmtsDataset)
        then DbSQLStmtsDataset.Free;
     FDatabase.ReadOnly  := WasReadOnly;
     FDatabase.Connected := WasConnected;
   end;
   end;
end;

procedure TSqlitePassSqlStmts.CreateSQLStmt(Const SQLStmtName: String; Const SQLStmt: TStringList);
begin
UpdateSQLStmtTable(0, SQLStmtName, '', SQLStmt);
end;

procedure TSqlitePassSqlStmts.RenameSQLStmt(Const OldName, NewName: String);
begin
UpdateSQLStmtTable(1, OldName, NewName, nil);
end;

procedure TSqlitePassSqlStmts.DeleteSQLStmt(Const SQLStmtName: String);
begin
UpdateSQLStmtTable(2, SQLStmtName, '', nil);
end;

function TSqlitePassSqlStmts.FindSQLStmt(const Value: string): TSqlitePassSqlStmt;
var
  I: Integer;
begin
  for I := 0 to Pred(Count) do
  begin
    Result := TSqlitePassSqlStmt(inherited Items[I]);
    if AnsiCompareText(Result.StmtName, Value) = 0 then Exit;
  end;
  Result := nil;
end;

function TSqlitePassSqlStmts.GetAttached: Boolean;
begin
 Result := (FAttachedDatabase <> nil);
end;

function TSqlitePassSqlStmts.GetItem(Index: Integer): TSqlitePassSqlStmt;
begin
Result:=TSqlitePassSqlStmt(Inherited Items[Index]);
end;

procedure TSqlitePassSqlStmts.Refresh;
const
Sql = 'SELECT * FROM "' + SqlitePassSQLStmtsTable + '"';
var
i: integer;
SQLStmtsRecordset: TSqlitePassRecordset;
SQLStmtsTableName: String;

  Procedure GetSQLStmts(AliasName: String; DatabaseConnection: Pointer);
  var
  i: integer;
  SQLStmt: TSqlitePassSQLStmt;
  begin
  if AliasName <> ''
     then SQLStmtsTableName := AliasName + '.' + SqlitePassSQLStmtsTable
     else SQLStmtsTableName := SqlitePassSQLStmtsTable;

  If (FDatabase.TableDefs.FindTable(SQLStmtsTableName) = nil) then Exit;

  SQLStmtsRecordset.Open(Sql);
  for i := 0 to Pred(SQLStmtsRecordset.count)
   do begin
      SQLStmt := TSqlitePassSQLStmt.Create(Self);
      SQLStmt.FStmtName := SQLStmtsRecordset[i].FieldsValues[0];
      SQLStmt.FSQL.Text := SQLStmtsRecordset[i].FieldsValues[1];
      Self.Add(SQLStmt);
      end;
  SQLStmtsRecordset.Close;
  end;

begin
Self.ClearAndFreeItems;
Try
  SQLStmtsRecordset:= TSqlitePassRecordset.Create(FDatabase);
  GetSQLStmts('', FDatabase.Engine.Connection);
  { Retrieves SQLStmts info for attached databases }
  For i := 0 to Pred(FDatabase.Databases.Count) do
      begin
      FDatabase.Databases[i].FEngine.OpenDatabase(FDatabase.Databases[i].Database);
      GetSQLStmts(FDatabase.Databases[i].AliasName, FDatabase.Databases[i].FEngine.Connection);
      FDatabase.Databases[i].FEngine.CloseDatabase;
      end;
finally
  SQLStmtsRecordset.Free;
end;
end;

procedure TSqlitePassSqlStmts.SetItem(Index: Integer;
  const Value: TSqlitePassSqlStmt);
begin
  Put(Index,Value);
end;

function TSqlitePassSqlStmts.SQLStmtByName(
  const Value: string): TSqlitePassSqlStmt;
begin
  Result := FindSQLStmt(Value);
  if Result = nil then
    DatabaseErrorFmt(Msg3950, [Value], FDatabase);
end;


