{ This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  ---------------------------------------------------------------------------

    Basic sqlite library wrapper for use with fpc compiler
    Author : Luc DAVID Email: luckylazarus@free.fr
    Dowload : http
    portions of code from Mike Cariotoglou Email: mike@deltasingular.gr
    2006 - 2008

    Major changes are indicated in the \Documentation\Changes.pdf file
    Last update 17.03.2008

  --------------------------------------------------------------------------- }

{ ----- Utilities ----- }
(*function _ExecCallback(cbParam: PSqlitePassExecCallBack; Columns: Integer;
                       ColumnValues, ColumnNames: Ppchar): integer; cdecl;
begin
try
// Result:=cbparam^(columns,columnValues,columnNames);
except
 result:=1;
end;
end; *)


{ TSqliteEngine }

constructor TSqlitePassEngine.Create(Database: TSqlitePassDatabase);
begin
FDatabase := Database;
FLibraryLoaded := False;
FLibraryHandle := 0;
FConnectionHandle:= nil;
FQueryTimeOut := 0;
FTransaction := TSqlitePassTransaction.Create(Self);
{$IFDEF DEBUG}
FPreparedStmts := TStringList.Create;
{$ENDIF}
end;

destructor TSqlitePassEngine.Destroy;
begin
CloseDatabase;
{$IFDEF DEBUG}
FPreparedStmts.Free;
{$ENDIF}
FTransaction.Free;
FDatabase := nil;
inherited Destroy;
end;

function TSqlitePassEngine.CheckResult(i: Integer; ShowMessage: Boolean = True): Boolean;
var
Msg: String;
begin
Result := (i in [SQLITE_OK, SQLITE_DONE]);
If (Not Result) then
   begin
   Msg := SqliteDbv3_Errmsg(FConnectionHandle);
   FDatabase.FDatabaseError.RaiseExceptionFmt('%d:%s',[i,Msg], i, FDatabase, ShowMessage);
   end;
end;

function TSqlitePassEngine.ExecSQL(Sql: string; CallBackFunc: TSqlitePassExecCallBack): Integer;
begin
StartChrono;
repeat
 (*if Assigned(CallBackFunc)
    then result:= SqliteDbv3_Exec(FConnectionHandle,Pchar(Sql),@_ExecCallBack,@@CallBackFunc,nil)
    else*)
Result:= SqliteDbv3_Exec(FConnectionHandle,Pchar(Sql),nil,nil,nil);
until (result=SQLITE_OK) or not (result in [SQLITE_BUSY,SQLITE_LOCKED])
     or ((FQueryTimeOut>=0) and (Integer(GetTickCount)>=FStartTime));
StopChrono;
CheckResult(Result);
end;

function TSqlitePassEngine.ExecQuery(Sql: string): Integer;
var
PSql, PRemainingSQL: Pchar;
DataPtr: Pointer;
begin
 StartChrono;
 PSql := PChar(Sql);
 DataPtr := nil;
try
 Repeat
 if CheckResult(SqliteDbv3_prepare_v2(FConnectionHandle,PSql,-1, DataPtr, PRemainingSQL))
    then Repeat Until CheckResult(SqliteDbv3_step(DataPtr), False);

    { If we have another sql stmt, we continu }
    if PRemainingSQL^<>#0 then
       begin
       CheckResult(SqliteDbv3_finalize(DataPtr));
       PSql:=PRemainingSQL;
       end;

 Until (PRemainingSQL^= #0);
finally
CheckResult(SqliteDbv3_Finalize(DataPtr));
StopChrono;
end;
end;

procedure TSqlitePassEngine.OpenDatabase(FullName, LibraryFile: String);
begin
FLibraryLoaded := LoadSqliteLibrary(LibraryFile);
CheckResult(SqliteDbv3_Open(Pchar(FullName),FConnectionHandle));
end;

procedure TSqlitePassEngine.CloseDatabase;
begin
If Assigned(FConnectionHandle) then
   begin
   Transaction.CommitAll;
   {$IFDEF DEBUG}
   FinalizePendingStmts;
   {$ENDIF}
   if CheckResult(SqliteDbv3_Close(FConnectionHandle)) then
      begin
      FConnectionHandle := nil;
      Dec(EnginesCount);
      end;
   end;
if FLibraryLoaded and (EnginesCount = 0) then
   begin
   FreeLibrary(FLibraryHandle);
   FLibraryLoaded := False;
   end;
end;

Function TSqlitePassEngine.LoadSqliteLibrary(LibraryFile: String): Boolean;
begin
Result := False;
if LibraryFile = ''
   then LibraryFile := DefaultSqliteLibrary;

{ The library is already loaded and we try to load the same file again}
if FLibraryLoaded and (LibraryFile = FLibraryFile)
   then exit;

if FLibraryLoaded
   then FreeLibrary(FLibraryHandle);

FLibraryFile := LibraryFile;
{$IFDEF WIN32}
FLibraryHandle := LoadLibrary(PChar(FLibraryFile));
{$ELSE}
FLibraryHandle := LoadLibrary(FLibraryFile);
{$ENDIF}

Result := FLibraryHandle <> 0;
if Result then Try
               LoadFunctions(FLibraryHandle);
               Except
               Result := False;
               end
          else FDatabase.DatabaseError.RaiseExceptionFmt(Msg3001, [FLibraryFile]);
end;

function TSqlitePassEngine.PrepareStmt(var Stmt: pointer; const sql: String): Boolean;
var
{$IFDEF DEBUG}
i: Integer;
{$ENDIF}

RemainingSql: PChar;
begin
 UnprepareStmt(Stmt);
 Result := CheckResult(SqliteDbv3_prepare_v2(FConnectionHandle,Pchar(Sql),-1,Stmt,RemainingSql));
 {$IFDEF DEBUG}
 If Result then
    begin
    i := FPreparedStmts.IndexOfName(IntToStr(Integer(Stmt)));
    if i > -1
       then FPreparedStmts.Strings[i] := IntToStr(Integer(Stmt)) + '=' + Sql
       else FPreparedStmts.Add(IntToStr(Integer(Stmt)) + '=' + Sql);
    end;
 {$ENDIF}
end;

function TSqlitePassEngine.UnprepareStmt(var Stmt: pointer): Boolean;
{$IFDEF DEBUG}
var
i: Integer;
{$ENDIF}
begin
Result := True;
if FConnectionHandle = nil then exit;
if Assigned(Stmt)
   then begin
        try
        Result := CheckResult(SqliteDbv3_Finalize(Stmt));
        {$IFDEF DEBUG}
        i := FPreparedStmts.IndexOfName(IntToStr(Integer(Stmt)));
        if i > -1
           then FPreparedStmts.Delete(i);
        {$ENDIF}
        finally
        Stmt := nil;
        end;
        end;
end;

{ TODO : Workaround to cleanup unused 'garbage' statements
  These statements should not exist, but...I have got to check my coding... }
procedure TSqlitePassEngine.FinalizePendingStmts;
var
Stmt: Pointer;
begin
While FPreparedStmts.Count > 0 do
  begin
  Stmt := Pointer(StrToInt(FPreparedStmts.Names[0]));
  UnPrepareStmt(Stmt);
  end;
end;

(*procedure TSqlitePassEngine.bindParamByName(const fst pointer; const param: tparam);
var idx:integer;
begin
 idx:=SqliteDbv3_bind_parameter_index(fst,pchar(param.Name));
 if idx=0 then raise exception.createFmt('parameter not found : %s',[param.Name]);
 BindParam(param,idx-1);
end;*)

{ TSqlitPassTransaction }

constructor TSqlitePassTransaction.Create(Owner: TSqlitePassEngine);
begin
inherited Create;
FEngine := Owner;
FLevel := 0;
end;

destructor TSqlitePassTransaction.Destroy;
begin
FEngine := nil;
Inherited Destroy;
end;

procedure TSqlitePassTransaction.Start;
begin
FEngine.ExecSql('Begin;');
Inc(FLevel);
end;

procedure TSqlitePassTransaction.Commit;
begin
FEngine.ExecSql('Commit;');
Dec(FLevel);
end;

procedure TSqlitePassTransaction.CommitAll;
begin
ResetTransactions(True);
end;

procedure TSqlitePassTransaction.Rollback;
begin
FEngine.ExecSql('Rollback;');
Dec(FLevel);
end;

procedure TSqlitePassTransaction.RollbackAll;
begin
ResetTransactions(False);
end;

procedure TSqlitePassTransaction.ResetTransactions(CommitTransaction: Boolean);
begin
While FLevel > 0 do
    Case CommitTransaction of
         True  : Commit;
         False : Rollback;
         end;
end;

procedure TSqlitePassTransaction.StartInternalTransaction;
begin
if FLevel = 0
   then begin
        FInternalTransaction := True;
        Start;
        end;
end;

procedure TSqlitePassTransaction.CommitInternalTransaction;
begin
if FInternalTransaction = True
   then begin
        FInternalTransaction := False;
        Commit;
        end;
end;

procedure TSqlitePassTransaction.RollbackInternalTransaction;
begin
if FInternalTransaction = True
   then begin
        FInternalTransaction := False;
        Rollback;
        end;
end;

procedure TSqlitePassEngine.StartChrono;
begin
 FStartTime:=Integer(GetTickCount);
end;

procedure TSqlitePassEngine.StopChrono;
begin
 FStopTime:=Integer(GetTickCount);
 FElapsedTime := (FStopTime - FStartTime) Div 1000;
end;

